﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Indlæsning {
    /*
     * medlemsliste = medlem {medlem}.
     *       medlem = navn fødselsår [kategori] semikolon.
     *     kategori = junior | senior.
    */
    public class MedlemslisteAnalysator {
        // Konstanter
        // Briktyper - enumerator (heltal i intervallet [0; 6])
        public enum Briktype {Navn, Fødselsår, Junior, Senior, Semikolon, Slut, Andet};
        // Konstante leksemer (terminaler)       
        public const string JUNIOR_LEKSEM = "junior";
        public const string SENIOR_LEKSEM = "senior";
        public const string SEMIKOLON_LEKSEM = ";";
        // Separatorer - lig whitespaces plus det grammatiske tegn ";" (semikolon)
        public const string SEPARATORER = " \t\n\f\r" + SEMIKOLON_LEKSEM;
        
        // Fields
        private StringTokenizer tokenizer; // Opdeler strengen i brikker
        private Brik brik; // Den aktuelle brik
        
        // Konstruktør
        public MedlemslisteAnalysator(string input) {
            // Initialiserer tokenizeren med input og separatorer
            // true: separatorerne returneres som tokens ...
            this.tokenizer = new StringTokenizer(input, SEPARATORER, true);
            brik = Brik.Instans; // Hent singleton-objekt
        }
        
        // Returnerer næste brik, (type + leksem)
        public void Next() {   
            // Spring over eventuelle "whitespaces" og find brikken
            bool found = true;
            while(this.tokenizer.HasMoreTokens()) {
                this.brik.Leksem = this.tokenizer.NextToken();
                if (!Char.IsWhiteSpace(this.brik.Leksem[0])) {
                    found = true;
                    break;
                }
            }
            if (found) { // Der er indlæst en brik ...
                // Terminaler (konstante leksemer)
                if (this.brik.Leksem.Equals(SEMIKOLON_LEKSEM)) {
                    this.brik.Type = (int)Briktype.Semikolon;
                    return;
                }
                if (this.brik.Leksem.Equals(JUNIOR_LEKSEM)) {
                    this.brik.Type = (int)Briktype.Junior;
                    return;
                }
                if (this.brik.Leksem.Equals(SENIOR_LEKSEM)) {
                    this.brik.Type = (int)Briktype.Senior;
                    return;
                }
                // Non-terminaler:
                // Er brikken Fødselsår?
                try {
                    if (int.Parse(this.brik.Leksem) > 0 && this.brik.Leksem.Length == 4) { 
                        this.brik.Type = (int)Briktype.Fødselsår;
                        return;
                    }
                }
                catch {} // Der var ikke et årstal ...
                // Er brikken Navn?
                int index = 0;
                while(index < brik.Leksem.Length) {
                    if (!Char.IsLetter(brik.Leksem[index]))
                        break;
                    index++;
                }
                if (index == brik.Leksem.Length) {
                    this.brik.Type = (int)Briktype.Navn;
                    return;
                }
                // Så må brikken være Andet
                this.brik.Type = (int)Briktype.Andet;
                return;
            }
            else { // Hvis der ikke er indlæst nogen brik, så ...
                this.brik.Type = (int)Briktype.Slut;
                return;
            }
        }
    }
}
