using System;
using System.Collections.Generic;
using System.Text;
using Indlsning;

namespace Regelbaseret {
    /*
     * medlemsliste = medlem {medlem}.
     *       medlem = navn fdselsr [kategori] semikolon.
     *     kategori = junior | senior.
    */
    // Klassen parser hele inputstrengen ved kald af metoden ParseMedlemsliste
    // Klassen benytter sig af Tabelstyret indlsning
    public class MedlemslisteParser {
        // Konstanter
        private const int SENIOR_ALDER = 18;
        // Fields
        private string resultat; // Streng, der indeholder parse-resultat
        private MedlemslisteAnalysator analysator; // Leksikals analysator
        // Konstruktr
        public MedlemslisteParser(string input) {
            // Allokering af leksikalsk analysator
            this.analysator = new MedlemslisteAnalysator(input);
            // Ls frste brik ...
            this.analysator.Next();
        }

        // Metoden parser hele inputstrengen. Parseren opbygger t medlem
        // ad gangen, (kald af metoden parseMedlem), og laver "kode-generering"
        // af hvert medlem, (kald af metoden Udskriv p "kodegeneratoren".

        // REGEL 1:  medlemsliste = medlem {medlem}
        public string ParseMedlemsliste() { // REGEL 7: medlem {medlem}
            do {
                if (!this.parseMedlem()) // REGEL 3: parseMedlem()
                    return this.resultat;
                Generator.Udskriv(Medlem.Instans);
            } while(Brik.Instans.Type == (int)MedlemslisteAnalysator.Briktype.Navn); // REGEL 5: Medlem
            // Hndtr afslutning
            if (Brik.Instans.Type == (int)MedlemslisteAnalysator.Briktype.Slut)
                this.resultat = "Indlsning slut. Ingen fejl!";
            else
                this.resultat = string.Format("Fejl i '{0}'. Slut eller navn forventet", Brik.Instans.Leksem);
            return resultat;
        }
        // Metoden parser en inputstreng for et medlem
        // REGEL 1: medlem = navn fdselsr [kategori] semikolon
        // Metoden opbygger et medlemsobjekt undervejs ...
        private bool parseMedlem() { // REGEL 7: navn fdselsr [kategori] semikolon
            // REGEL 2: navn
            if (Brik.Instans.Type == (int)MedlemslisteAnalysator.Briktype.Navn) {
                Medlem.Instans.Navn = Brik.Instans.Leksem;
                this.analysator.Next();
            }
            else {
                this.resultat = string.Format("Fejl i '{0}'. Navn forventet", Brik.Instans.Leksem);
                return false;
            }
            // REGEL 2: fdselsr
            if (Brik.Instans.Type == (int)MedlemslisteAnalysator.Briktype.Fdselsr) {
                Medlem.Instans.Fdselsr = int.Parse(Brik.Instans.Leksem);
                this.analysator.Next();
            }
            else {
                this.resultat = string.Format("Fejl i '{0}'. Fdselsr forventet", Brik.Instans.Leksem);
                return false;
            }
            Medlem.Instans.Kategori = "";
            //REGEL 4: [kategori]
            if (Brik.Instans.Type == (int)MedlemslisteAnalysator.Briktype.Junior || Brik.Instans.Type == (int)MedlemslisteAnalysator.Briktype.Senior) {
                if (!parseKategori()) // REGEL 3: parseKategori()
                    return false;
            }
            // REGEL 2: semikolon
            if (Brik.Instans.Type == (int)MedlemslisteAnalysator.Briktype.Semikolon) {
                this.analysator.Next();
            }
            else {
                this.resultat = string.Format("Fejl i '{0}'. Semikolon forventet", Brik.Instans.Leksem);
                return false;
            }
            return true;
        }
        // Metoden parser inputstreng for en kategori
        // REGEL 3: kategori = junior | senior
        private bool parseKategori() {
            int alder = DateTime.Now.Year - Medlem.Instans.Fdselsr;
            // REGEL 6: kategori
            switch (Brik.Instans.Type) {
                case (int)MedlemslisteAnalysator.Briktype.Junior:
                    // REGEL 2: junior
                    // Sematisk check af konsistens mellem alder og kategori
                    if (alder < SENIOR_ALDER) {
                        Medlem.Instans.Kategori = Brik.Instans.Leksem;
                        this.analysator.Next();
                        return true;
                    }
                    else {
                        this.resultat = string.Format("Fejl i '{0}'. Fdselsr og kategori stemmer ikke overens!", Brik.Instans.Leksem);
                        return false;
                    }
                case (int)MedlemslisteAnalysator.Briktype.Senior:
                    // REGEL 2: junior
                    // Sematisk check af konsistens mellem alder og kategori
                    if (alder >= SENIOR_ALDER) {
                        Medlem.Instans.Kategori = Brik.Instans.Leksem;
                        this.analysator.Next();
                        return true;
                    }
                    else {
                        this.resultat = string.Format("Fejl i '{0}'. Fdselsr og kategori stemmer ikke overens!", Brik.Instans.Leksem);
                        return false;
                    }
            }
            return true;
        }
    }
}
