using System;
using System.Collections.Generic;
using System.Text;
using Indlsning;

namespace Tabelstyret {
    /*
     * medlemsliste = medlem {medlem}.
     *       medlem = navn fdselsr [kategori] semikolon.
     *     kategori = junior | senior.
    */
    // Klassen parser hele inputstrengen ved kald af metoden ParseMedlemsliste
    // Klassen benytter sig af Tabelstyret indlsning
    public class MedlemslisteParser {
        // Konstanter og enumeratorer
        private const int SENIOR_ALDER = 18;
        private const int ANTAL_TILSTANDE = 7;
        private const int ANTAL_BRIKKER = 6;
        private enum tilstande { start, efterNavn, efterFdselsr, efterKategori, efterSemikolon, slut, fejl };
        private enum aktioner { navn, fdselsr, kategori, semikolon, slut, fejl };
        // Fields
        private string resultat; // Streng, der indeholder parse-resultat
        private MedlemslisteAnalysator analysator; // Leksikals analysator
        // Tilstandstabel og Aktionstabel
        private int[,] tilstandstabel;
        private int[,] aktionstabel;
        private int aktuelTilstand;
        // Konstruktr
        public MedlemslisteParser(string input) {
            // Allokering leksikalsk analysator
            this.analysator = new MedlemslisteAnalysator(input);
            
            // Allokering af tilstands- og aktionstabel
            this.tilstandstabel = new int[ANTAL_TILSTANDE, ANTAL_BRIKKER];
            this.aktionstabel = new int[ANTAL_TILSTANDE, ANTAL_BRIKKER];
            
            // Gennemlb tabellerne og nulstil tilstandstabel med fejltilstande og
            // nulstil aktionstabel med fejlaktioner
            for(int i = 0; i < ANTAL_TILSTANDE; i++) {
                for (int j = 0; j < ANTAL_BRIKKER; j++) {
                    this.tilstandstabel[i, j] = (int)tilstande.fejl;
                    this.aktionstabel[i, j] = (int)aktioner.fejl;
                }
            }

            // St tilladelige tilstande
            this.tilstandstabel[(int)tilstande.start, (int)MedlemslisteAnalysator.Briktype.Navn] = (int)tilstande.efterNavn;
            this.tilstandstabel[(int)tilstande.efterNavn, (int)MedlemslisteAnalysator.Briktype.Fdselsr] = (int)tilstande.efterFdselsr;
            this.tilstandstabel[(int)tilstande.efterFdselsr, (int)MedlemslisteAnalysator.Briktype.Junior] = (int)tilstande.efterKategori;
            this.tilstandstabel[(int)tilstande.efterFdselsr, (int)MedlemslisteAnalysator.Briktype.Senior] = (int)tilstande.efterKategori;
            this.tilstandstabel[(int)tilstande.efterFdselsr, (int)MedlemslisteAnalysator.Briktype.Semikolon] = (int)tilstande.efterSemikolon;
            this.tilstandstabel[(int)tilstande.efterKategori, (int)MedlemslisteAnalysator.Briktype.Semikolon] = (int)tilstande.efterSemikolon;
            this.tilstandstabel[(int)tilstande.efterSemikolon, (int)MedlemslisteAnalysator.Briktype.Navn] = (int)tilstande.efterNavn;
            this.tilstandstabel[(int)tilstande.efterSemikolon, (int)MedlemslisteAnalysator.Briktype.Slut] = (int)tilstande.slut;
            
            // St aktioner for skift til tilladelige tilstande
            this.aktionstabel[(int)tilstande.start, (int)MedlemslisteAnalysator.Briktype.Navn] = (int)aktioner.navn;
            this.aktionstabel[(int)tilstande.efterNavn, (int)MedlemslisteAnalysator.Briktype.Fdselsr] = (int)aktioner.fdselsr;
            this.aktionstabel[(int)tilstande.efterFdselsr, (int)MedlemslisteAnalysator.Briktype.Junior] = (int)aktioner.kategori;
            this.aktionstabel[(int)tilstande.efterFdselsr, (int)MedlemslisteAnalysator.Briktype.Senior] = (int)aktioner.kategori;
            this.aktionstabel[(int)tilstande.efterFdselsr, (int)MedlemslisteAnalysator.Briktype.Semikolon] = (int)aktioner.semikolon;
            this.aktionstabel[(int)tilstande.efterKategori, (int)MedlemslisteAnalysator.Briktype.Semikolon] = (int)aktioner.semikolon;
            this.aktionstabel[(int)tilstande.efterSemikolon, (int)MedlemslisteAnalysator.Briktype.Navn] = (int)aktioner.navn;
            this.aktionstabel[(int)tilstande.efterSemikolon, (int)MedlemslisteAnalysator.Briktype.Slut] = (int)aktioner.slut;
            
            // St aktuel tilstand til starttilstand
            this.aktuelTilstand = (int)tilstande.start;
        }

        // Metoden parser hele inputstrengen. Parseren opbygger t medlem
        // ad gangen, (kald af metoden parseMedlem), og laver "kode-generering"
        // af hvert medlem, (kald af metoden Udskriv p "kodegeneratoren".

        public string ParseMedlemsliste() {
            int aktion;
            do {
                this.analysator.Next();
                // Bestem aktion
                aktion = this.aktionstabel[this.aktuelTilstand, Brik.Instans.Type];   
                // Udfr aktion (semantik)
                switch (aktion) {
                    case (int)aktioner.navn: // Gem navn
                        Medlem.Instans.Navn = Brik.Instans.Leksem;
                        break;
                    case (int)aktioner.fdselsr: // Gem fdselsr
                        Medlem.Instans.Fdselsr = int.Parse(Brik.Instans.Leksem);
                        break;
                    case (int)aktioner.kategori:  // Check og gem kategori
                        int alder = DateTime.Now.Year - Medlem.Instans.Fdselsr;
                        if ((Brik.Instans.Type == (int)MedlemslisteAnalysator.Briktype.Junior && alder < SENIOR_ALDER) ||
                            (Brik.Instans.Type == (int)MedlemslisteAnalysator.Briktype.Senior && alder >= SENIOR_ALDER))
                            Medlem.Instans.Kategori = Brik.Instans.Leksem;
                        else
                            return string.Format("Fejl i '{0}'. Fdselsr og kategori stemmer ikke overens!", Brik.Instans.Leksem);
                        break;
                    case (int)aktioner.semikolon: // Kald "kodegenerator"
                        Generator.Udskriv(Medlem.Instans);
                        Medlem.Instans.Kategori = "";
                        break;
                    case (int)aktioner.slut: // Ingen aktion
                        this.resultat = this.resultat = "Indlsning slut. Ingen fejl!";
                        break;
                    case (int)aktioner.fejl: // Lav fejltekst
                        this.resultat = string.Format("Fejl i '{0}'", Brik.Instans.Leksem);
                        break;
                }
                // Bestem ny tilstand
                this.aktuelTilstand = this.tilstandstabel[this.aktuelTilstand, Brik.Instans.Type];
            } while (this.aktuelTilstand != (int)tilstande.fejl && this.aktuelTilstand != (int)tilstande.slut);
            return this.resultat;
        }
    }
}
